@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark.c,v 1.21 2003/12/14 12:33:20 ikezoe Exp $
+ *  $Id: kz-bookmark.c,v 1.35 2004/04/12 15:07:45 ikezoe Exp $
  */
 
 #include "kz-bookmark.h"
@@ -28,15 +28,16 @@
 #include "intl.h"
 #include "egg-marshalers.h"
 
-#include "kz-downloader.h"
 #include "kz-io.h"
 
 /* bookmark file types */
 #include "kz-xbel.h"
 #include "kz-rss.h"
+#include "kz-nsbookmark.h"
 #include "kz-lirs.h"
 #include "kz-hinadi.h"
 #include "kz-google.h"
+#include "kz-scrap-bookmark.h"
 
 enum {
 	INSERT_CHILD_SIGNAL,
@@ -47,6 +48,7 @@ enum {
 	LOAD_COMPLETED_SIGNAL,
 	SAVE_START_SIGNAL,
 	SAVE_COMPLETED_SIGNAL,
+	ERROR_SIGNAL,
 	LAST_SIGNAL
 };
 
@@ -90,21 +92,17 @@ static void kz_bookmark_real_insert_child (KzBookmark *bookmark,
 static void kz_bookmark_real_remove_child (KzBookmark *bookmark,
 					   KzBookmark *child);
 
-static void kz_bookmark_set_state          (KzBookmark *bookmark,
-					    KzBookmarkState state);
+static void kz_bookmark_set_state         (KzBookmark *bookmark,
+					   KzBookmarkState state);
+static void kz_bookmark_set_editable      (KzBookmark *bookmark,
+					   gboolean    editable);
 
-/* load/save */
-static void kz_bookmark_load_completed   (KzBookmark   *bookmark,
-					  KzDownloader *dl,
-					  gboolean      error);
+static void cb_io_load_complete           (KzIO *io,
+					   GError *error,
+					   KzBookmark   *bookmark);
 static KzBookmarkFileType *
-            kz_bookmark_detect_file_type (KzBookmark   *bookmark,
-					  gchar        *buf);
-static void cb_downloader_load_complete  (KzDownloader *dl,
-					  KzBookmark   *bookmark);
-static void cb_downloader_load_error     (KzDownloader *dl,
-					  KzBookmark   *bookmark);
-
+            kz_bookmark_detect_file_type  (KzBookmark   *bookmark,
+					   gchar        *buf);
 static GObjectClass *parent_class = NULL;
 static gint kz_bookmark_signals[LAST_SIGNAL] = {0};
 
@@ -160,6 +158,8 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 		ADD_BOOKMARK_FILE_TYPE(kz_hinadi_get_file_types);
 		ADD_BOOKMARK_FILE_TYPE(kz_google_get_file_types);
 		ADD_BOOKMARK_FILE_TYPE(kz_xbel_get_file_types);
+		ADD_BOOKMARK_FILE_TYPE(kz_nsbookmark_get_file_types);
+		ADD_BOOKMARK_FILE_TYPE(kz_scrap_bookmark_get_file_types);
 	}
 
 	parent_class = g_type_class_peek_parent (klass);
@@ -363,12 +363,21 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 			       g_cclosure_marshal_VOID__VOID,
 			       G_TYPE_NONE, 0);
 
+	kz_bookmark_signals[ERROR_SIGNAL]
+		= g_signal_new("error",
+			       G_TYPE_FROM_CLASS(klass),
+			       G_SIGNAL_RUN_LAST,
+			       G_STRUCT_OFFSET (KzBookmarkClass,
+						error),
+			       NULL, NULL,
+			       g_cclosure_marshal_VOID__STRING,
+			       G_TYPE_NONE, 1, G_TYPE_STRING);
+
 	/* FIXME! add other properties */
 	/*
 	 * candidates:
 	 * 
 	 *   * ID
-	 *   * Last-Modified time
 	 *   * Last-Accessed time
 	 *   * frequency
 	 *   * ......
@@ -475,7 +484,13 @@ kz_bookmark_set_property (GObject *object,
 			if (type && type->init)
 				type->init(bookmark);
 			if (type && !type->to_string)
+			{
 				kz_bookmark_set_editable(bookmark, FALSE);
+				/*
+				 * FIXME!!
+				 * Also set all children as non-editable.
+				 */
+			}
 		}
 		break;
 	}
@@ -648,6 +663,8 @@ kz_bookmark_file_new (const gchar *location,
 {
 	KzBookmark *bookmark;
 
+	location = location ? location : "";
+
 	bookmark = KZ_BOOKMARK(g_object_new(KZ_TYPE_BOOKMARK,
 					    "type",      KZ_BOOKMARK_FOLDER,
 					    "location",  location,
@@ -669,6 +686,13 @@ kz_bookmark_file_create_new (const gchar *location,
 	KzBookmark *bookmark;
 	KzBookmarkFileType *type;
 
+	if (!location)
+	{
+		g_warning(_("kz_bookmark_file_create_new(): "
+			    "location is not specified!"));
+		location = "";
+	}
+
 	bookmark = KZ_BOOKMARK(g_object_new(KZ_TYPE_BOOKMARK,
 					    "type",      KZ_BOOKMARK_FOLDER,
 					    "location",  location,
@@ -758,8 +782,38 @@ kz_bookmark_get_interval (KzBookmark *bookmark)
 gboolean
 kz_bookmark_is_editable (KzBookmark *bookmark)
 {
+	KzBookmark *parent_file;
+
 	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), FALSE);
-	return bookmark->flags & KZ_BOOKMARK_EDITABLE_FLAG ? TRUE : FALSE;
+
+	parent_file = kz_bookmark_get_parent_file(bookmark);
+
+	if (!KZ_IS_BOOKMARK(parent_file))
+		return FALSE;
+
+	if (!kz_bookmark_file_is_editable(parent_file))
+		return FALSE;
+
+	return TRUE;
+}
+
+
+gboolean
+kz_bookmark_file_is_editable (KzBookmark *bookmark_file)
+{
+	KzBookmarkFileType *type;
+
+	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark_file), FALSE);
+	g_return_val_if_fail(kz_bookmark_is_file(bookmark_file), FALSE);
+
+	if (!kz_bookmark_get_file_type(bookmark_file))
+		return FALSE;
+
+	type = kz_bookmark_detect_file_type(bookmark_file, NULL);
+	if (type && type->to_string)
+		return TRUE;
+
+	return FALSE;
 }
 
 
@@ -854,7 +908,7 @@ kz_bookmark_is_separator (KzBookmark *bookmark)
 }
 
 
-void
+static void
 kz_bookmark_set_editable (KzBookmark *bookmark, gboolean editable)
 {
 	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
@@ -966,7 +1020,10 @@ kz_bookmark_prepend (KzBookmark *bookmark, KzBookmark *child)
 	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
 
 	list = g_object_get_qdata(G_OBJECT(bookmark), children_quark);
-	kz_bookmark_insert_before(bookmark, child, list->data);
+	if (list)
+		kz_bookmark_insert_before(bookmark, child, list->data);
+	else
+		kz_bookmark_insert_before(bookmark, child, NULL);		
 }
 
 
@@ -986,7 +1043,7 @@ kz_bookmark_real_remove_child (KzBookmark *bookmark, KzBookmark *child)
 	g_object_unref(child);
 	g_object_set_qdata(G_OBJECT(bookmark), children_quark, list);
 
-	g_object_set_qdata(G_OBJECT(child), parent_quark, NULL);
+	g_object_set_qdata(G_OBJECT(child), parent_quark, NULL);	
 }
 
 
@@ -1030,6 +1087,24 @@ kz_bookmark_get_parent (KzBookmark *bookmark)
 }
 
 
+KzBookmark *
+kz_bookmark_get_parent_file (KzBookmark *bookmark)
+{
+	KzBookmark *parent = bookmark;
+
+	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), NULL);
+
+	do
+	{
+		parent = kz_bookmark_get_parent(parent);
+		if (KZ_IS_BOOKMARK(parent) && kz_bookmark_is_file(parent))
+			return parent;
+	} while (parent);
+
+	return NULL;
+}
+
+
 GList *
 kz_bookmark_get_children (KzBookmark *bookmark)
 {
@@ -1153,7 +1228,7 @@ gboolean
 kz_bookmark_load_start (KzBookmark *bookmark)
 {
 	KzBookmarkState state;
-	KzDownloader *dl;
+	KzIO *io;
 	const gchar *uri;
 
 	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), FALSE);
@@ -1172,14 +1247,11 @@ kz_bookmark_load_start (KzBookmark *bookmark)
 
 	kz_bookmark_remove_all(KZ_BOOKMARK(bookmark));
 
-	dl = kz_downloader_new(uri);
-	g_signal_connect(G_OBJECT(dl), "completed",
-			 G_CALLBACK(cb_downloader_load_complete),
+	io = kz_io_new(uri);
+	g_signal_connect(G_OBJECT(io), "io_completed",
+			 G_CALLBACK(cb_io_load_complete),
 			 bookmark);
-	g_signal_connect(G_OBJECT(dl), "error",
-			 G_CALLBACK(cb_downloader_load_error),
-			 bookmark);
-	kz_downloader_to_string(dl);
+	kz_io_load_to_buffer(io);
 
 	return TRUE;
 }
@@ -1233,9 +1305,10 @@ kz_bookmark_save (KzBookmark *bookmark)
 
 	if (file && *file && str && *str)
 	{
-		io = kz_io_new(file, KZ_IO_WRITE);
-		kz_io_set_string(io, str);
-		kz_io_start(io);
+		io = kz_io_new(file);
+		kz_io_write(io, str);
+		g_object_unref(io);
+		g_free(str);
 	}
 }
 
@@ -1243,34 +1316,33 @@ kz_bookmark_save (KzBookmark *bookmark)
 static gboolean
 idle_load_complete(gpointer data)
 {
-	KzDownloader *dl = data;
+	KzIO *io = data;
 
-	g_object_unref(G_OBJECT(dl));
+	g_object_unref(G_OBJECT(io));
 
 	return FALSE;
 }
 
 
 static void
-kz_bookmark_load_completed(KzBookmark *bookmark, KzDownloader *dl,
-			   gboolean error)
-{
+cb_io_load_complete(KzIO *io, GError *error, KzBookmark *bookmark)
+{	
 	KzBookmarkFileType *type;
-	gchar *buf = NULL;
+	const gchar *buf;
 	guint size;
 
 	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
-	g_return_if_fail(KZ_IS_DOWNLOADER(dl));
+	g_return_if_fail(KZ_IS_IO(io));
 
 	if (error) goto ERROR;
 
-	buf  = kz_downloader_get_string(dl);
-	size = kz_downloader_get_current_progress(dl);
+	buf  = kz_io_get_buffer(io);
+	size = kz_io_get_loaded_size(io);
 
 	g_return_if_fail(buf != NULL);
 	if (size == 0) goto ERROR;
 
-	type = kz_bookmark_detect_file_type(bookmark, buf);
+	type = kz_bookmark_detect_file_type(bookmark, (gchar *)buf);
 	if (!type)
 		/* use default? */;
 	if (type && type->from_string)
@@ -1281,21 +1353,27 @@ kz_bookmark_load_completed(KzBookmark *bookmark, KzDownloader *dl,
 	}
 
 ERROR:
-	g_free(buf);
 
 	g_signal_handlers_disconnect_by_func
-		(G_OBJECT(dl),
-		 G_CALLBACK(cb_downloader_load_complete),
-		 bookmark);
-	g_signal_handlers_disconnect_by_func
-		(G_OBJECT(dl),
-		 G_CALLBACK(cb_downloader_load_error),
+		(G_OBJECT(io),
+		 G_CALLBACK(cb_io_load_complete),
 		 bookmark);
 	kz_bookmark_set_state(bookmark, KZ_BOOKMARK_STATE_NORMAL);
-	g_signal_emit(G_OBJECT (bookmark),
-		      kz_bookmark_signals[LOAD_COMPLETED_SIGNAL],
-		      0);
-	g_idle_add(idle_load_complete, dl);
+
+	if (error)
+	{
+		g_signal_emit(G_OBJECT (bookmark),
+			      kz_bookmark_signals[ERROR_SIGNAL],
+			      0,
+			      "Load failed");
+	}
+	else 
+	{
+		g_signal_emit(G_OBJECT (bookmark),
+			      kz_bookmark_signals[LOAD_COMPLETED_SIGNAL],
+			      0);
+	}
+	g_idle_add(idle_load_complete, io);
 }
 
 
@@ -1338,20 +1416,6 @@ kz_bookmark_detect_file_type(KzBookmark *bookmark, gchar *buf)
 }
 
 
-static void
-cb_downloader_load_complete(KzDownloader *dl, KzBookmark *bookmark)
-{	
-	kz_bookmark_load_completed(bookmark, dl, FALSE);
-}
-
-
-static void
-cb_downloader_load_error(KzDownloader *dl, KzBookmark *bookmark)
-{	
-	kz_bookmark_load_completed(bookmark, dl, TRUE);
-}
-
-
 static void
 kz_bookmark_set_state (KzBookmark *bookmark, KzBookmarkState state)
 {
