@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmarks-view.c,v 1.5 2004/03/25 04:32:23 makeinu Exp $
+ *  $Id: kz-bookmarks-view.c,v 1.7 2004/04/13 11:47:00 ikezoe Exp $
  */
 
 #include "kz-bookmarks-view.h"
@@ -556,6 +556,7 @@ insert_bookmark(KzBookmarksView *view,
 							 NULL);
 			gtk_widget_destroy(image);
 		}
+		g_object_unref(kz_favicon);
 	}
 
 	if (view->priv->editable)
@@ -932,7 +933,7 @@ cb_bookmark_location_edited (GtkCellRendererText *cell,
 	if (kz_bookmark_is_file(bookmark))
 	{
 		kz_bookmark_set_location(bookmark, new_text);
-		kz_bookmark_load_in_memory_start(bookmark);
+		kz_bookmark_load_start(bookmark);
 	}
 	else
 	{
