@@ -17,13 +17,15 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-xbel.c,v 1.26 2003/12/17 04:22:22 ikezoe Exp $
+ *  $Id: kz-xbel.c,v 1.38 2004/04/12 15:07:45 ikezoe Exp $
  */
 
+#include "kazehakase.h"
 #include "kz-xbel.h"
 #include "kz-xml.h"
 
 #include <string.h>
+#include <stdlib.h>
 
 static gboolean kz_xbel_is_supported       (KzBookmark  *bookmark,
 					    const gchar *buf);
@@ -39,13 +41,6 @@ static void     kz_xbel_notify             (GObject     *object,
 					    KzXML       *xml);
 
 static void     kz_xbel_build_tree         (KzBookmark  *bookmark);
-static void     kz_xbel_insert_child       (KzBookmark  *bookmark,
-					    KzBookmark  *child,
-					    KzBookmark  *sibling,
-					    KzXML       *xml);
-static void     kz_xbel_remove_child       (KzBookmark  *bookmark,
-					    KzBookmark  *child,
-					    KzXML       *xml);
 static void     kz_xbel_insert_xml_node    (KzBookmark  *bookmark,
 					    KzBookmark  *parent,
 					    KzBookmark  *sibling);
@@ -144,10 +139,10 @@ kz_xbel_init (KzBookmark *bookmark)
 
 	g_signal_connect(G_OBJECT(bookmark), "notify",
 			 G_CALLBACK(kz_xbel_notify), xml);
-	g_signal_connect(G_OBJECT(bookmark), "insert-child",
-			 G_CALLBACK(kz_xbel_insert_child), xml);
-	g_signal_connect(G_OBJECT(bookmark), "remove-child",
-			 G_CALLBACK(kz_xbel_remove_child), xml);
+	g_signal_connect_after(G_OBJECT(bookmark), "insert-child",
+			       G_CALLBACK(cb_bookmark_insert_child), xml);
+	g_signal_connect_after(G_OBJECT(bookmark), "remove-child",
+			       G_CALLBACK(cb_bookmark_remove_child), xml);
 }
 
 
@@ -169,12 +164,18 @@ kz_xbel_from_string (KzBookmark *bookmark,
 		KzXMLNode *node, *title_node, *title, *space;
 		const gchar *bookmark_title;
 
+		/*
+		 *  <xbel version=1.0 folded=none>\n (space1)
+		 *  <title>Bookmarks</title>\n       (space2)
+		 *  </xbel>
+		 */
 		node = kz_xml_element_node_new("xbel");
 		kz_xml_node_set_attr(node, "version", "1.0");
 		kz_xml_node_set_attr(node, "folded", "none");	
 		/* kz_xml_node_set_attr(node, "id", "????????"); */
 		kz_xml_node_append_child(xml->root, node);
 
+		/* space1 */
 		space = kz_xml_text_node_new("\n");
 		kz_xml_node_append_child(node, space);
 
@@ -190,6 +191,7 @@ kz_xbel_from_string (KzBookmark *bookmark,
 			kz_xml_node_append_child(title_node, title);
 		}
 
+		/* space2 */
 		space = kz_xml_text_node_new("\n");
 		kz_xml_node_append_child(node, space);
 	}
@@ -270,6 +272,158 @@ kz_xbel_notify (GObject *object, GParamSpec *pspec, KzXML *xml)
 }
 
 
+static void
+parse_metadata_node (KzBookmark *bookmark, KzXMLNode *parent)
+{
+	KzXMLNode *node;
+
+	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
+
+	if (!kz_xml_node_is_element(parent)) return;
+
+	for (node = kz_xml_node_first_child(parent);
+	     node;
+	     node = kz_xml_node_next(node))
+	{
+		if (!kz_xml_node_is_element(node)) continue;
+
+		if (kz_xml_node_name_is(node, "metadata"))
+		{
+			const gchar *owner;
+
+			/* We should ignore other owners such as Galeon. */
+			owner = kz_xml_node_get_attr(node, "owner");
+			if (!owner || strcmp(owner, KAZEHAKASE_URI))
+				continue;
+
+			parse_metadata_node(bookmark, node);
+			/*
+			if (kz_bookmark_get_owner(bookmark))
+			{
+				g_warning("metadata owner attribute is duplicated!");
+				continue;
+			}
+			owner = kz_xml_node_get_attr(node, "owner");
+			kz_bookmark_set_owner(folder, owner);
+			*/
+		}
+		else if (kz_xml_node_name_is(node, "location"))
+		{
+			/*
+			 * "location" element is already parsed at
+			 * parse_child_node().
+			 */
+			if (!kz_bookmark_is_file(bookmark))
+			{
+				g_warning("kzXBEL: The bookmark has location, "
+					  "but it's not a folder element!");
+			}
+		}
+		else if (kz_xml_node_name_is(node, "update_interval"))
+		{
+			gchar *interval_str;
+			guint interval;
+
+			if (kz_bookmark_get_interval(bookmark))
+			{
+				g_warning("update_interval element is duplicated!");
+				continue;
+			}
+			interval_str = kz_xml_node_to_str(node);
+			interval = atoi(interval_str);
+			kz_bookmark_set_interval(bookmark, interval);
+			g_free(interval_str);
+		}
+	}	
+}
+
+
+static KzXMLNode *
+xml_node_get_named_node (KzXMLNode *parent, const gchar *name)
+{
+	KzXMLNode *node;
+
+	g_return_val_if_fail(parent, NULL);
+	g_return_val_if_fail(name && *name, NULL);
+
+	for (node = kz_xml_node_first_child(parent);
+	     node;
+	     node = kz_xml_node_next(node))
+	{
+		if (kz_xml_node_name_is(node, name))
+			return node;
+	}
+
+	return NULL;
+}
+
+
+static KzXMLNode *
+xml_node_find_metadata_node (KzXMLNode *parent)
+{
+	KzXMLNode *info_node, *node;
+
+	info_node = xml_node_get_named_node(parent, "info");
+	if (!info_node) return NULL;
+
+	for (node = kz_xml_node_first_child(info_node);
+	     node;
+	     node = kz_xml_node_next(node))
+	{
+		if (kz_xml_node_name_is(node, "metadata"))
+		{
+			const gchar *owner;
+
+			owner = kz_xml_node_get_attr(node, "owner");
+			if (owner && !strcmp(owner, KAZEHAKASE_URI))
+				return node;
+		}
+	}
+
+	return NULL;
+}
+
+
+static gboolean
+xml_node_has_location (KzXMLNode *parent)
+{
+	KzXMLNode *child, *gchild;
+
+	g_return_val_if_fail(parent, FALSE);
+
+	if (!kz_xml_node_name_is(parent, "folder"))
+		return FALSE;
+
+	child = xml_node_find_metadata_node(parent);
+	if (!child) return FALSE;
+
+	gchild = xml_node_get_named_node(child, "location");
+	if (gchild) return TRUE;
+
+	return FALSE;
+}
+
+
+static gchar *
+xml_node_get_location (KzXMLNode *parent)
+{
+	KzXMLNode *child, *gchild;
+
+	g_return_val_if_fail(parent, FALSE);
+
+	if (!kz_xml_node_name_is(parent, "folder"))
+		return NULL;
+
+	child = xml_node_find_metadata_node(parent);
+	if (!child) return NULL;
+
+	gchild = xml_node_get_named_node(child, "location");
+	if (!gchild) return NULL;
+
+	return kz_xml_node_to_str(gchild);
+}
+
+
 static void
 parse_child_node (KzBookmark *bookmark, KzXMLNode *parent)
 {
@@ -318,6 +472,20 @@ parse_child_node (KzBookmark *bookmark, KzXMLNode *parent)
 			const gchar *folded_str;
 			gboolean folded;
 
+			if (xml_node_has_location(node))
+			{
+#warning FIXME! If the file does not exit, we should creat it.
+				gchar *location = xml_node_get_location(node);
+
+				folder = kz_bookmark_file_new(location,
+							      NULL, NULL);
+				kz_bookmark_load_start(folder);
+			}
+			else
+			{
+				folder = kz_bookmark_pure_folder_new();
+			}
+
 			folded_str = kz_xml_node_get_attr(node, "folded");
 			folded = folded_str && !strcmp(folded_str, "yes");
 			kz_bookmark_set_folded(folder, folded);
@@ -328,6 +496,13 @@ parse_child_node (KzBookmark *bookmark, KzXMLNode *parent)
 			kz_bookmark_append(bookmark, folder);
 			parse_child_node(folder, node);
 			BOOKMARK_UNSET_BUILDING(folder);
+
+			if (kz_bookmark_is_file(folder))
+			{
+				kz_bookmark_remove_all(folder);
+				kz_bookmark_load_start(folder);
+			}
+
 			g_object_unref(folder);
 		}
 		else if (kz_xml_node_name_is(node, "bookmark"))
@@ -360,6 +535,10 @@ parse_child_node (KzBookmark *bookmark, KzXMLNode *parent)
 			g_warning("KzXBEL::alias element is not supported yet."
 				  "Please implement me!");
 		}
+		else if (kz_xml_node_name_is(node, "info"))
+		{
+			parse_metadata_node(bookmark, node);
+		}
 	}
 }
 
@@ -393,9 +572,9 @@ xml_node_append_title (KzXMLNode *node, KzBookmark *bookmark)
 	const gchar *title = kz_bookmark_get_title(bookmark);
 	KzXMLNode *title_node, *text_node, *space;
 
-	/* space */
-	space = kz_xml_text_node_new("\n");
-	kz_xml_node_append_child(node, space);
+	/*
+	 *  <title>title</title>\n
+	 */
 
 	/* title */
 	title_node = kz_xml_element_node_new("title");
@@ -403,16 +582,149 @@ xml_node_append_title (KzXMLNode *node, KzBookmark *bookmark)
 	text_node = kz_xml_text_node_new(title);
 	kz_xml_node_append_child(title_node, text_node);
 
-	/* space */
+	/* space1 */
 	space = kz_xml_text_node_new("\n");
 	kz_xml_node_append_child(node, space);
 }
 
 
+static KzXMLNode *
+xml_node_append_metadata_node (KzXMLNode *node)
+{
+	KzXMLNode *space, *info_node, *metadata_node;
+
+	/*
+	 *  <info>\n      (space1)
+	 *  <metadata>\n  (space2)
+	 *  </metadata>\n (space3)
+	 *  </info>\n     (space4)
+	 */
+
+	/* info */
+	info_node = xml_node_get_named_node(node, "info");
+	if (!info_node)
+	{
+		info_node = kz_xml_element_node_new("info");
+		kz_xml_node_append_child(node, info_node);
+
+		/* space1 */
+		space = kz_xml_text_node_new("\n");
+		kz_xml_node_append_child(info_node, space);
+
+		/* space4 */
+		space = kz_xml_text_node_new("\n");
+		kz_xml_node_append_child(node, space);
+	}
+
+	/* metadata */
+	metadata_node = xml_node_find_metadata_node(node);
+	if (metadata_node) return metadata_node;
+
+	metadata_node = kz_xml_element_node_new("metadata");
+	kz_xml_node_set_attr(metadata_node, "owner", KAZEHAKASE_URI);
+	kz_xml_node_append_child(info_node, metadata_node);
+
+	/* space2 */
+	space = kz_xml_text_node_new("\n");
+	kz_xml_node_append_child(metadata_node, space);
+
+	/* space3 */
+	space = kz_xml_text_node_new("\n");
+	kz_xml_node_append_child(info_node, space);
+
+	return metadata_node;
+}
+
+
+static void
+xml_node_set_location (KzXMLNode *node, KzBookmark *bookmark)
+{
+	const gchar *location;
+	KzXMLNode *metadata_node, *location_node,
+		  *text_node, *space;
+
+	location = kz_bookmark_get_location(bookmark);
+	g_return_if_fail(location);
+
+	metadata_node = xml_node_append_metadata_node(node);
+	location_node = xml_node_get_named_node(metadata_node, "location");
+
+	if (location_node)
+	{
+		KzXMLNode *child;
+
+		/* remove all child node */
+		child = kz_xml_node_first_child(location_node);
+		while (child)
+		{
+			KzXMLNode *next = kz_xml_node_next(child);
+			kz_xml_node_remove_child(location_node, child);
+			kz_xml_node_unref(child);
+			child = next;
+		}
+	}
+	else
+	{
+		location_node = kz_xml_element_node_new("location");
+		kz_xml_node_append_child(metadata_node, location_node);
+	}
+
+	text_node = kz_xml_text_node_new(location);
+	kz_xml_node_append_child(location_node, text_node);
+
+	space = kz_xml_text_node_new("\n");
+	kz_xml_node_append_child(metadata_node, space);
+}
+
+
+static void
+xml_node_set_interval (KzXMLNode *node, KzBookmark *bookmark)
+{
+	gchar *interval;
+	KzXMLNode *metadata_node, *interval_node,
+		  *text_node, *space;
+
+	interval = g_strdup_printf("%d", kz_bookmark_get_interval(bookmark));
+
+	metadata_node = xml_node_append_metadata_node(node);
+	interval_node = xml_node_get_named_node(metadata_node,
+						"update_interval");
+
+	/* update_interval */
+	if (interval_node)
+	{
+		KzXMLNode *child;
+
+		/* remove all child node */
+		child = kz_xml_node_first_child(interval_node);
+		while (child)
+		{
+			KzXMLNode *next = kz_xml_node_next(child);
+			kz_xml_node_remove_child(interval_node, child);
+			kz_xml_node_unref(child);
+			child = next;
+		}
+	}
+	else
+	{
+		interval_node = kz_xml_element_node_new("update_interval");
+		kz_xml_node_append_child(metadata_node, interval_node);
+	}
+
+	text_node = kz_xml_text_node_new(interval);
+	kz_xml_node_append_child(interval_node, text_node);
+
+	space = kz_xml_text_node_new("\n");
+	kz_xml_node_append_child(metadata_node, space);
+
+	g_free(interval);
+}
+
+
 static KzXMLNode *
 create_xml_node (KzBookmark *bookmark)
 {
-	KzXMLNode *node = NULL;
+	KzXMLNode *node = NULL, *space;
 
 	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), NULL);
 
@@ -420,18 +732,67 @@ create_xml_node (KzBookmark *bookmark)
 	{
 		node = kz_xml_element_node_new("separator");
 	}
+	else if (kz_bookmark_is_file(bookmark))
+	{
+		const gchar *location = kz_bookmark_get_location(bookmark);
+		guint interval        = kz_bookmark_get_interval(bookmark);
+
+		if (!location || !*location)
+		{
+			/* warning? */
+		}
+
+		/*
+		 * <folder>\n
+		 * <title/>
+		 * ...
+		 * </folder>
+		 */
+
+		node = kz_xml_element_node_new("folder");
+		xml_node_append_title (node, bookmark);
+
+		space = kz_xml_text_node_new("\n");
+		kz_xml_node_append_child(node, space);
+
+		if (location)
+			xml_node_set_location(node, bookmark);
+		if (interval)
+			xml_node_set_interval(node, bookmark);
+	}
 	else if (kz_bookmark_is_folder(bookmark))
 	{
+		/*
+		 * <folder>\n
+		 * <title/>
+		 * ...
+		 * </folder>
+		 */
+
 		node = kz_xml_element_node_new("folder");
+
+		space = kz_xml_text_node_new("\n");
+		kz_xml_node_append_child(node, space);
+
 		xml_node_append_title (node, bookmark);
 	}
 	else
 	{
 		const gchar *uri = kz_bookmark_get_link(bookmark);
 
+		/*
+		 * <bookmark>\n
+		 * <title/>
+		 * </bookmark>
+		 */
+
 		node = kz_xml_element_node_new("bookmark");
 		if (uri)
 			kz_xml_node_set_attr(node, "href", uri);
+
+		space = kz_xml_text_node_new("\n");
+		kz_xml_node_append_child(node, space);
+
 		xml_node_append_title (node, bookmark);
 	}
 
@@ -442,18 +803,22 @@ create_xml_node (KzBookmark *bookmark)
 static void
 kz_xbel_connect_signals (KzBookmark *bookmark)
 {
-	if (kz_bookmark_is_folder(bookmark))
-	{
-		g_signal_connect(G_OBJECT(bookmark), "insert-child",
-				 G_CALLBACK(cb_bookmark_insert_child), NULL);
-		g_signal_connect(G_OBJECT(bookmark), "remove-child",
-				 G_CALLBACK(cb_bookmark_remove_child), NULL);
-	}
-
 	g_signal_connect(G_OBJECT(bookmark), "notify",
 			 G_CALLBACK(cb_bookmark_notify), NULL);
 
-	if (kz_bookmark_is_folder(bookmark))
+	if (kz_bookmark_is_file(bookmark)) return;
+	if (!kz_bookmark_is_folder(bookmark)) return;
+
+	{
+		g_signal_connect_after(G_OBJECT(bookmark), "insert-child",
+				       G_CALLBACK(cb_bookmark_insert_child),
+				       NULL);
+		g_signal_connect_after(G_OBJECT(bookmark), "remove-child",
+				       G_CALLBACK(cb_bookmark_remove_child),
+				       NULL);
+	}
+
+	/* for children */
 	{
 		GList *node, *children;
 
@@ -471,7 +836,13 @@ kz_xbel_connect_signals (KzBookmark *bookmark)
 static void
 kz_xbel_disconnect_signals (KzBookmark *bookmark)
 {
-	if (kz_bookmark_is_folder(bookmark))
+	g_signal_handlers_disconnect_by_func
+		(G_OBJECT(bookmark),
+		 G_CALLBACK(cb_bookmark_notify), NULL);
+
+	if (kz_bookmark_is_file(bookmark)) return;
+	if (!kz_bookmark_is_folder(bookmark)) return;
+
 	{
 		g_signal_handlers_disconnect_by_func
 			(G_OBJECT(bookmark),
@@ -481,11 +852,7 @@ kz_xbel_disconnect_signals (KzBookmark *bookmark)
 			 G_CALLBACK(cb_bookmark_remove_child), NULL);
 	}
 
-	g_signal_handlers_disconnect_by_func
-		(G_OBJECT(bookmark),
-		 G_CALLBACK(cb_bookmark_notify), NULL);
-
-	if (kz_bookmark_is_folder(bookmark))
+	/* for children */
 	{
 		GList *node, *children;
 
@@ -502,7 +869,7 @@ kz_xbel_disconnect_signals (KzBookmark *bookmark)
 
 static void
 cb_bookmark_insert_child (KzBookmark *bookmark,
-		 KzBookmark *child, KzBookmark *sibling)
+			  KzBookmark *child, KzBookmark *sibling)
 {
 	kz_xbel_insert_xml_node(child, bookmark, sibling);
 	kz_xbel_connect_signals(child);
@@ -604,6 +971,59 @@ xml_node_set_description (KzXMLNode *parent, const gchar *desc)
 }
 
 
+#if 0
+static void
+xml_node_set_element (KzXMLNode *parent,
+		      const gchar *contents, const gchar *name)
+{
+	KzXMLNode *element_node = NULL, *node, *child;
+
+	g_return_if_fail(parent);
+	g_return_if_fail(name);
+
+	node = kz_xml_node_first_child(parent);
+
+	/* find node */
+	for (; node; node = kz_xml_node_next(node))
+	{
+		if (kz_xml_node_name_is(node, name))
+		{
+			element_node = node;
+			break;
+		}
+	}
+
+	/* remove old element node, or create new element node */
+	if (element_node)
+	{
+		child = kz_xml_node_first_child(node);
+		while(child)
+		{
+			KzXMLNode *next = kz_xml_node_next(child);
+
+			child = kz_xml_node_remove_child(node, child);
+			kz_xml_node_unref(child);
+			child = next;
+		}
+	}
+	else
+	{
+		KzXMLNode *space;
+
+		element_node = kz_xml_element_node_new(name);
+		kz_xml_node_append_child(parent, element_node);
+
+		space = kz_xml_text_node_new("\n");
+		kz_xml_node_append_child(parent, space);
+	}
+
+	/* set new element */
+	child = kz_xml_text_node_new(contents);
+	kz_xml_node_append_child(element_node, child);
+}
+#endif
+
+
 static void
 cb_bookmark_notify (GObject *object, GParamSpec *pspec)
 {
@@ -652,22 +1072,16 @@ cb_bookmark_notify (GObject *object, GParamSpec *pspec)
 		xml_node_set_description(node, desc);
 		g_free(desc);
 	}
-}
-
-
-static void
-kz_xbel_insert_child (KzBookmark *bookmark,
-		      KzBookmark *child, KzBookmark *sibling,
-		      KzXML *xml)
-{
-	cb_bookmark_insert_child(bookmark, child, sibling);
-}
-
-
-static void
-kz_xbel_remove_child (KzBookmark *bookmark, KzBookmark *child, KzXML *xml)
-{
-	cb_bookmark_remove_child(bookmark, child);
+	else if (!strcmp(prop, "location"))
+	{
+		xml_node_set_location(node, bookmark);
+		/* xml_node_set_metadata(node, "location", str) */
+	}
+	else if (!strcmp(prop, "interval"))
+	{
+		xml_node_set_interval(node, bookmark);
+		/* xml_node_set_metadata(node, "update_interval", str); */
+	}
 }
 
 
@@ -729,7 +1143,7 @@ kz_xbel_insert_xml_node (KzBookmark *bookmark,
 	space = kz_xml_text_node_new("\n");
 	kz_xml_node_insert_before(parent_node, space, kz_xml_node_next(node));
 
-	if (kz_bookmark_is_folder(bookmark))
+	if (kz_bookmark_is_folder(bookmark) && !kz_bookmark_is_file(bookmark))
 	{
 		GList *node, *children;
 
