@@ -21,44 +21,29 @@
 #ifndef __GOOGLE_H__
 #define __GOOGLE_H__
 
+/*
+ *  This widget is temporary solution to create a google item.
+ *  We'll replace this with KzBookmark and KzBookmarkAction.
+ *  (Smart bookmark)
+ */
+
 #include <gtk/gtk.h>
-#include <kz-window.h>
+#include "kz-window.h"
+#include "kz-bookmark.h"
 
 G_BEGIN_DECLS
 
 typedef struct _Google Google;
-typedef struct _SearchResult SearchResult;
 
 struct _Google
 {
 	KzWindow *kz;
 	/* widget */
-	GtkWidget *KeywordEntry;
-	GtkWidget *Icon;
-	
-	/* google uri */
-	gchar *uri;
-	
-	/* keyword history */
-	GList *keyword;
-	
-	/* serch result */
-	GSList *results;
-};
-
-struct _SearchResult
-{
-	/* keyword */
-	gchar *keyword;
-	/* local file name */
-	gchar *localfile;
-	/* result uri */
-	gchar *uri;
-	/* menu Widget */
-	GtkWidget *ResultMenu;
+	GtkWidget *entry;
 };
 
 GtkWidget *google_new (KzWindow *kz);
+Google    *google_get (GtkWidget *widget);
 
 G_END_DECLS
 
