@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-item.c,v 1.8 2004/01/27 08:19:05 makeinu Exp $
+ *  $Id: kz-bookmark-item.c,v 1.12 2004/04/12 15:07:44 ikezoe Exp $
  */
 
 /*
@@ -38,7 +38,7 @@
 #include "kz-icons.h"
 #include "kz-actions.h"
 #include "gtk-utils.h"
-#include "eel-input-event-box.h"
+#include "kz-input-event-box.h"
 
 enum {
 	PROP_0,
@@ -70,9 +70,12 @@ static void     open_all_items                    (KzBookmark *bookmark,
 						   KzWindow *kz);
 
 /* callback for bookmark objects */
-static void     cb_bookmark_load_start            (KzBookmark *meta,
+static void     cb_bookmark_load_start            (KzBookmark *bookmark,
 						   KzBookmarkItem *menu);
-static void     cb_bookmark_load_stop             (KzBookmark *meta,
+static void     cb_bookmark_load_stop             (KzBookmark *bookmark,
+						   KzBookmarkItem *menu);
+static void     cb_bookmark_load_error            (KzBookmark *bookmark,
+						   const gchar *error,
 						   KzBookmarkItem *menu);
 static void     bookmark_weak_ref                 (gpointer data,
 						   GObject *obj);
@@ -157,33 +160,70 @@ kz_bookmark_item_constructor (GType                  type,
 	GObject *object;
 	GObjectClass *klass = G_OBJECT_CLASS(parent_class);
 	KzBookmarkState state;
+	gboolean is_file, is_folder;
 
 	object = klass->constructor(type, n_props, props);
 
 	hbox = GTK_BOX(object);
 	menu = KZ_BOOKMARK_ITEM(object);
-
+	is_file = kz_bookmark_is_file(menu->bookmark);
+	is_folder = kz_bookmark_is_folder(menu->bookmark);
+	
 	gtk_container_set_border_width(GTK_CONTAINER(menu), 2);
 
 	/* button */
 	/* FIXME!: show favicon? */
 	menu->update_button = button = gtk_button_new();
 	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-	g_signal_connect(G_OBJECT(button), "clicked", 
-			 G_CALLBACK(cb_update_button_clicked),
-			 menu);
-
 	gtk_box_pack_start(hbox, button, FALSE, FALSE, 0);
-	menu->image = image
-		= gtk_image_new_from_stock(KZ_STOCK_GREEN,
-					   KZ_ICON_SIZE_BOOKMARK_MENU);
+	if (is_file)
+	{
+		g_signal_connect(G_OBJECT(button), "clicked", 
+				 G_CALLBACK(cb_update_button_clicked),
+				 menu);
+		menu->image = image
+			= gtk_image_new_from_stock(KZ_STOCK_GREEN,
+						   KZ_ICON_SIZE_BOOKMARK_MENU);
+	}
+	else if(is_folder)
+	{
+		g_signal_connect(G_OBJECT(button), "button_press_event",
+			 	 G_CALLBACK(cb_bookmark_menu_press), menu);
+		menu->image = image
+			= gtk_image_new_from_stock(KZ_STOCK_FOLDER,
+						   KZ_ICON_SIZE_BOOKMARK_MENU);
+	}
+	else
+	{
+		KzFavicon *kzfav;
+		GtkWidget *favicon = NULL;
+
+		g_signal_connect(G_OBJECT(button), "button_press_event",
+			 	 G_CALLBACK(cb_bookmark_menu_press), menu);
+		/* favicon */
+		kzfav = kz_favicon_get_instance();
+		favicon	= kz_favicon_get_widget(kzfav,
+						kz_bookmark_get_link(menu->bookmark),
+						KZ_ICON_SIZE_BOOKMARK_MENU);
+		if (favicon)
+		{
+			menu->image = image = favicon;
+		}
+		else
+		{
+			menu->image = image
+				= gtk_image_new_from_stock(GTK_STOCK_NEW,
+							   KZ_ICON_SIZE_BOOKMARK_MENU);
+		}
+		g_object_unref(kzfav);
+	}
 	gtk_container_add(GTK_CONTAINER(button), image);
 	gtk_widget_show(image);
 	gtk_widget_show(button);
 	style = gtk_rc_style_new ();
 	style->xthickness = style->ythickness = 0;
 	gtk_widget_modify_style (button, style);
-	gtk_rc_style_unref (style),
+	gtk_rc_style_unref (style);
 
 	g_signal_connect(G_OBJECT(menu->bookmark),
 			 "load_start", 
@@ -193,13 +233,17 @@ kz_bookmark_item_constructor (GType                  type,
 			 "load_completed", 
 			 G_CALLBACK(cb_bookmark_load_stop),
 			 menu);
+	g_signal_connect(G_OBJECT(menu->bookmark),
+			 "error", 
+			 G_CALLBACK(cb_bookmark_load_error),
+			 menu);
 
 	state = kz_bookmark_get_state(menu->bookmark);
 	if (state == KZ_BOOKMARK_STATE_LOADING)
 		cb_bookmark_load_start(menu->bookmark, menu);
 
 	/* label */
-	eventbox = eel_input_event_box_new();
+	eventbox = kz_input_event_box_new();
 	menu->eventbox = eventbox;
 	gtk_box_pack_start(hbox, eventbox, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(eventbox), "button_press_event",
@@ -241,6 +285,10 @@ kz_bookmark_item_dispose (GObject *object)
 			G_OBJECT(menu->bookmark),
 			cb_bookmark_load_stop,
 			menu);
+		g_signal_handlers_disconnect_by_func(
+			G_OBJECT(menu->bookmark),
+			cb_bookmark_load_error,
+			menu);
 		g_object_unref(menu->bookmark);
 		menu->bookmark = NULL;
 	}
@@ -374,9 +422,9 @@ open_all_items (KzBookmark *bookmark, KzWindow *kz)
  *                                                                           *
  *****************************************************************************/
 static void 
-cb_bookmark_load_start (KzBookmark *meta, KzBookmarkItem *menu)
+cb_bookmark_load_start (KzBookmark *bookmark, KzBookmarkItem *menu)
 {
-	g_return_if_fail(KZ_IS_BOOKMARK(meta));
+	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
 	g_return_if_fail(KZ_IS_BOOKMARK_ITEM(menu));
 
 	gtk_image_set_from_stock(GTK_IMAGE(menu->image),
@@ -399,6 +447,18 @@ cb_bookmark_load_stop (KzBookmark *bookmark, KzBookmarkItem *menu)
 }
 
 
+static void 
+cb_bookmark_load_error (KzBookmark *bookmark, const gchar *error, KzBookmarkItem *menu)
+{
+	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
+	g_return_if_fail(KZ_IS_BOOKMARK_ITEM(menu));
+
+	gtk_image_set_from_stock(GTK_IMAGE(menu->image),
+				 KZ_STOCK_GRAY,
+				 KZ_ICON_SIZE_BOOKMARK_MENU);
+}
+
+
 static void
 cb_notify_title (GObject *object, GParamSpec *pspec,
 		 KzBookmarkItem *menu)
@@ -474,7 +534,7 @@ cb_bookmark_menu_press (GtkWidget *widget, GdkEventButton *event,
 		break;
 	}
 
-	return FALSE;
+	return TRUE;
 }
 
 
@@ -483,7 +543,7 @@ cb_update_button_clicked (GtkWidget *widget, KzBookmarkItem *menu)
 {
 	g_return_if_fail(KZ_IS_BOOKMARK_ITEM(menu));
 
-	kz_bookmark_load_in_memory_start(menu->bookmark);
+	kz_bookmark_load_start(menu->bookmark);
 }
 
 
