@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-menu.c,v 1.23 2004/01/25 14:11:57 ikezoe Exp $
+ *  $Id: kz-bookmark-menu.c,v 1.26 2004/03/20 15:30:37 tyuyu Exp $
  */
 
 #include "kz-bookmark-menu.h"
@@ -28,6 +28,7 @@
 #include "kz-bookmark-editor.h"
 #include "kz-actions.h"
 #include "kz-favicon.h"
+#include "utils.h"
 
 #define KZ_BOOKMARK_MENU_BOOKMARK_KEY "KzBookmarkMenu::Bookmark"
 #define KZ_BOOKMARK_MENU_WINDOW_KEY   "KzBookmarkMenu::Window"
@@ -67,6 +68,7 @@ kz_bookmark_menu_create_menuitem (KzBookmark *bookmark, KzWindow *kz,
 {
 	GtkWidget *menu_item;
 	const gchar *title = NULL, *desc;
+	gchar *tooltip_text = NULL;
 	GtkTooltips *tooltip = NULL;
 	GtkWidget *favicon;
 	KzFavicon *kzfav;
@@ -94,7 +96,9 @@ kz_bookmark_menu_create_menuitem (KzBookmark *bookmark, KzWindow *kz,
 	if (!title)
 		title = "";
 	desc = kz_bookmark_get_description(bookmark);
-
+	if (desc)
+		tooltip_text = remove_tag(desc, strlen(desc));
+	
 	menu_item = gtk_image_menu_item_new_with_label(title);
 	g_object_set_data(G_OBJECT(menu_item),
 			  KZ_BOOKMARK_MENU_BOOKMARK_KEY,
@@ -139,12 +143,15 @@ kz_bookmark_menu_create_menuitem (KzBookmark *bookmark, KzWindow *kz,
 	tooltip = gtk_tooltips_new();
 	g_object_ref(G_OBJECT(tooltip));
 	gtk_object_sink(GTK_OBJECT(tooltip));
-	gtk_tooltips_set_tip(tooltip, menu_item, desc, NULL);
+	if (tooltip_text)
+	{
+		gtk_tooltips_set_tip(tooltip, menu_item, tooltip_text, NULL);
+		g_free(tooltip_text);
+	}
 	g_object_set_data_full(G_OBJECT(menu_item),
 			       KZ_BOOKMARK_MENU_TOOLTIP_KEY,
 			       tooltip,
 			       (GDestroyNotify) g_object_unref);
-
 	connect_bookmark_signals(menu_item, bookmark);
 
 	return menu_item;
@@ -182,14 +189,14 @@ kz_bookmark_menu_create_submenu (KzBookmark *folder, KzWindow *kz)
 	gtk_widget_show(menu_item);
 
 	/* children */
-	kz_boommark_menu_append_menuitems(GTK_MENU_SHELL(submenu), kz, folder);
+	kz_bookmark_menu_append_menuitems(GTK_MENU_SHELL(submenu), kz, folder);
 
 	return submenu;
 }
 
 
 void
-kz_boommark_menu_append_menuitems (GtkMenuShell *shell,KzWindow *kz, 
+kz_bookmark_menu_append_menuitems (GtkMenuShell *shell,KzWindow *kz, 
 				   KzBookmark *folder)
 {
 	KzBookmark *prev_folder;
@@ -253,7 +260,7 @@ kz_boommark_menu_append_menuitems (GtkMenuShell *shell,KzWindow *kz,
 
 
 void
-kz_boommark_menu_remove_menuitems (GtkMenuShell *shell, KzWindow *kz)
+kz_bookmark_menu_remove_menuitems (GtkMenuShell *shell, KzWindow *kz)
 {
 	KzBookmark *folder;
 	GList *children, *node;
@@ -386,12 +393,18 @@ cb_notify_desc (GObject *object, GParamSpec *pspec,
 {
 	GtkTooltips *tooltip;
 	const gchar *desc;
-
+	gchar *tooltip_text = NULL;
+	
 	desc = kz_bookmark_get_description(KZ_BOOKMARK(object));
+	if (desc)
+		tooltip_text = remove_tag(desc, strlen(desc));
 	tooltip = g_object_get_data(G_OBJECT(widget),
 				    KZ_BOOKMARK_MENU_TOOLTIP_KEY);
-	if (GTK_IS_TOOLTIPS(tooltip))
-		gtk_tooltips_set_tip(tooltip, widget, desc, NULL);
+	if (GTK_IS_TOOLTIPS(tooltip) && tooltip_text)
+	{
+		gtk_tooltips_set_tip(tooltip, widget, tooltip_text, NULL);
+		g_free(tooltip_text);
+	}
 }
 
 
@@ -479,7 +492,7 @@ cb_menu_item_button_press (GtkWidget *widget, GdkEventButton *event,
 		{
 			GtkWidget *editor;
 
-			editor = kz_bookmark_editor_new(kz_bookmark);
+			editor = kz_bookmark_editor_new(kz_bookmarks->menu);
 			kz_bookmark_editor_folder_view_select
 				(KZ_BOOKMARK_EDITOR(editor), item);
 			gtk_widget_show(editor);
@@ -595,8 +608,8 @@ cb_folder_reordered (KzBookmark *item, GtkMenuShell *shell)
 	KzWindow *kz;
 
 	kz = g_object_get_data(G_OBJECT(shell), KZ_BOOKMARK_MENU_WINDOW_KEY);
-	kz_boommark_menu_remove_menuitems(shell, kz);
-	kz_boommark_menu_append_menuitems(shell, kz, item);
+	kz_bookmark_menu_remove_menuitems(shell, kz);
+	kz_bookmark_menu_append_menuitems(shell, kz, item);
 }
 
 
