@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-icons.h,v 1.5 2003/11/10 19:06:06 makeinu Exp $
+ *  $Id: kz-icons.h,v 1.8 2004/03/24 14:45:52 ikezoe Exp $
  */
 
 #ifndef KZ_ICONS_H
@@ -30,9 +30,12 @@ G_BEGIN_DECLS
 #define KZ_STOCK_GOOGLE       "kz-google"
 #define KZ_STOCK_GREEN        "kz-green"
 #define KZ_STOCK_RED          "kz-red"
+#define KZ_STOCK_GRAY         "kz-gray"
 #define KZ_STOCK_FOLDER       "kz-folder"
 #define KZ_STOCK_FOLDER_OPEN  "kz-folder-open"
 #define KZ_STOCK_SEPARATOR    "kz-separator"
+#define KZ_STOCK_ANCHOR       "kz-anchor"
+#define KZ_STOCK_BOOKMARK     "kz-bookmark"
 
 extern GtkIconSize KZ_ICON_SIZE_BOOKMARK_MENU;
 
