@@ -52,6 +52,7 @@
 #include "kz-bookmark-menu.h"
 #include "kz-paned.h"
 #include "kz-entry.h"
+#include "kz-proxy-menu.h"
 
 enum {
 	APPEND_TAB_SIGNAL,
@@ -144,12 +145,20 @@ static void     kz_window_set_moz_callbacks      (KzWindow   *kz,
 static void     kz_window_unset_moz_callbacks    (KzWindow   *kz,
 						  KzMozEmbed *kzembed);
 
+static void     kz_window_restore_session     (KzWindow  *kz);
+
 /* callbacks */
 static void     cb_profile_global_changed     (KzProfile       *profile,
 					       const gchar     *section,
 					       const gchar     *key,
 					       const gchar     *old_value,
 					       KzWindow        *kz);
+static void     cb_profile_proxy_changed      (KzProfile       *profile,
+					       const gchar     *section,
+					       const gchar     *key,
+					       const gchar     *old_value,
+					       KzWindow        *kz);
+
 static void     cb_profile_gesture_changed    (KzProfile       *profile,
 					       const gchar     *section,
 					       const gchar     *key,
@@ -188,8 +197,10 @@ static void     cb_sidebar_map                (GtkWidget       *widget,
 					       EggToggleAction *action);
 static void     cb_sidebar_unmap              (GtkWidget       *widget,
 					       EggToggleAction *action);
-
-
+static gboolean cb_focus_out_event            (GtkWidget       *widget,
+					       GdkEventFocus   *event,
+					       KzWindow        *kz);
+ 
 /* callbacks for mozilla */
 static void     cb_embed_title_changed         (GtkMozEmbed *embed,
 						KzWindow    *kz);
@@ -254,6 +265,7 @@ static void     cb_notebook_drag_data_received (GtkWidget        *widget,
 
 
 void kz_window_sync_ui_level (KzWindow *kz);
+void kz_window_sync_proxy (KzWindow *kz);
 
 
 static GtkWindowClass *parent_class = NULL;
@@ -399,6 +411,9 @@ kz_window_init (KzWindow *kz)
 		g_signal_connect(G_OBJECT(kz_global_profile),
 				 "changed::Gesture",
 				 G_CALLBACK(cb_profile_gesture_changed), kz);
+		g_signal_connect(GTK_WINDOW(kz),
+				 "focus-out-event",
+				 G_CALLBACK(cb_focus_out_event), kz);
 	}
 
 	/* top level vbox */
@@ -430,6 +445,10 @@ kz_window_init (KzWindow *kz)
 		g_signal_connect(G_OBJECT(kz_global_profile),
 				 "changed::Global",
 				 G_CALLBACK(cb_profile_global_changed), kz);
+		g_signal_connect(G_OBJECT(kz_proxy),
+				 "changed",
+				 G_CALLBACK(cb_profile_proxy_changed), kz);
+
 
 		/* FIXME */
 		setting = gtk_settings_get_default();
@@ -600,6 +619,8 @@ kz_window_init (KzWindow *kz)
 	kz_window_restore_state(kz);
 
 	gtk_widget_show(GTK_WIDGET(kz));
+
+	kz_window_restore_session(kz);
 }
 
 
@@ -718,10 +739,30 @@ kz_window_sync_ui_level (KzWindow *kz)
 						  kz_bookmarks->menu);
 	}
 
+        /* update proxy */
+	kz_window_sync_proxy(kz);
+
 	g_free(ui_file);
 	g_free(level);
 }
 
+void
+kz_window_sync_proxy (KzWindow *kz)
+{
+	gchar path[64] = {0};
+	GtkWidget *widget;
+
+	g_snprintf(path, sizeof(path), "/menu/EditMenu%s/ProxyMenu",
+		   kz_ui_level_to_suffix(kz_ui_level()));
+	widget = egg_menu_merge_get_widget(kz->menu_merge, path);
+	if (GTK_IS_MENU_ITEM(widget))
+		widget = gtk_menu_item_get_submenu(GTK_MENU_ITEM(widget));
+	if (GTK_IS_MENU_SHELL(widget))
+	{
+		kz_proxy_menu_remove_menuitems(GTK_MENU_SHELL(widget), kz);
+		kz_proxy_menu_append_menuitems(GTK_MENU_SHELL(widget), kz);
+	}
+}
 
 GtkWidget *
 kz_window_get_from_tab (GtkWidget *tab_widget)
@@ -751,11 +792,9 @@ kz_window_open_new_tab (KzWindow *kz, const gchar *url)
 	return kz_window_open_new_tab_with_parent (kz, url, NULL);
 }
 
-/*
- * This function is only called by restoring session.
- */
+
 GtkWidget *
-kz_window_open_new_tab_at_position (KzWindow *kz, const gchar *url, gint pos)
+kz_window_open_new_tab_at_tail (KzWindow *kz, const gchar *url)
 {
 	KzMozEmbed *kzembed;
 	KzTabLabel *kztab;
@@ -769,14 +808,12 @@ kz_window_open_new_tab_at_position (KzWindow *kz, const gchar *url, gint pos)
 	kz_favicon_connect_signal(kz->kzfav, kzembed);
 
 	kztab = KZ_TAB_LABEL(kz_tab_label_new(kz, kzembed));
-	
-	gtk_notebook_insert_page(GTK_NOTEBOOK(kz->notebook),
-				 GTK_WIDGET(kzembed),
-				 GTK_WIDGET (kztab),
-				 pos);			
 
 	gtk_widget_show(GTK_WIDGET(kzembed));
 	gtk_widget_show(GTK_WIDGET(kztab));
+	gtk_notebook_append_page(GTK_NOTEBOOK(kz->notebook),
+				 GTK_WIDGET(kzembed),
+				 GTK_WIDGET(kztab));
 	kz_window_set_moz_callbacks(kz, kzembed);
 
 	/* add to this window's history */
@@ -815,6 +852,8 @@ kz_window_open_new_tab_with_parent (KzWindow *kz, const gchar *url,
 
 	kztab = KZ_TAB_LABEL(kz_tab_label_new(kz, kzembed));
 
+	gtk_widget_show(GTK_WIDGET(kzembed));
+	gtk_widget_show(GTK_WIDGET(kztab));
 	position = KZ_CONF_GET_STR("Tab", "new_tab_position");
 	
 	if (!position || !strcasecmp(position, "last"))
@@ -878,8 +917,6 @@ kz_window_open_new_tab_with_parent (KzWindow *kz, const gchar *url,
 		}
 	}
 
-	gtk_widget_show(GTK_WIDGET(kzembed));
-	gtk_widget_show(GTK_WIDGET(kztab));
 	kz_window_set_moz_callbacks(kz, kzembed);
 
 	/* add to this window's history */
@@ -1181,6 +1218,9 @@ kz_window_destroy (GtkObject *object)
 	g_signal_handlers_disconnect_by_func
 		(G_OBJECT(kz_global_profile),
 		 G_CALLBACK(cb_profile_gesture_changed), kz);
+	g_signal_handlers_disconnect_by_func
+		(G_OBJECT(kz_proxy),
+		 G_CALLBACK(cb_profile_proxy_changed), kz);
 
 	if (!kz_window_get_window_list())
 	{
@@ -1453,13 +1493,6 @@ kz_window_restore_state (KzWindow *kz)
 					     TRUE);
 	}
 	g_free(tab_pos);
-
-
-	/*
-	 *  Session
-	 */
-
-	kz_window_restore_session(kz);
 }
 
 
@@ -1637,18 +1670,6 @@ kz_window_load_url (KzWindow *kz, const gchar *url)
 }
 
 
-void
-kz_window_load_home (KzWindow *kz)
-{
-	EggAction *action;
-
-	g_return_if_fail(KZ_IS_WINDOW(kz));
-
-	action = egg_action_group_get_action(kz->actions, "Home");
-	egg_action_activate(action);
-}
-
-
 const gchar *
 kz_window_get_title (KzWindow *kz)
 {
@@ -1719,10 +1740,36 @@ cb_profile_global_changed (KzProfile *profile,
 	g_return_if_fail(KZ_IS_PROFILE(profile));
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	if (key && !strcmp(key, "ui_level"))
-		kz_window_sync_ui_level(kz);
+	switch (key[0])
+	{
+	case 'u':/* ui_level */
+		if (strcmp(key, "ui_level") == 0)
+		{
+			kz_window_sync_ui_level(kz);
+		}
+		break;
+	case 'p':
+		if (strcmp(key, "proxy_name") == 0)
+		{
+			kz_window_sync_proxy(kz);
+		}
+		break;
+	default:
+		break;
+	}
 }
 
+static void
+cb_profile_proxy_changed (KzProfile *profile,
+			   const gchar *section, const gchar *key,
+			   const gchar *old_value,
+			   KzWindow *kz)
+{
+	g_return_if_fail(KZ_IS_PROFILE(profile));
+	g_return_if_fail(KZ_IS_WINDOW(kz));
+
+	kz_window_sync_proxy(kz);
+}
 
 static void
 cb_profile_gesture_changed (KzProfile *profile,
@@ -2637,3 +2684,26 @@ cb_notebook_drag_data_received (GtkWidget *widget,
 		break;
 	}
 }
+
+static gboolean 
+cb_focus_out_event            (GtkWidget       *widget,
+			       GdkEventFocus   *event,
+			       KzWindow        *kz)
+{
+	g_return_val_if_fail(KZ_IS_WINDOW(widget), FALSE);
+
+	if(!event->in) {/* focus is out */
+		if (kz_gesture_is_started(kz->priv->gesture))
+		{
+			kz_gesture_cancel(kz->priv->gesture);
+			gtk_statusbar_push(GTK_STATUSBAR(KZ_WINDOW(widget)->statusbar),
+				   STATUS_GESTURE, "");
+		}
+
+		if (gdk_pointer_is_grabbed ())
+		{
+			gdk_pointer_ungrab (gtk_get_current_event_time ());
+		}
+	}
+	return FALSE;
+}
