@@ -23,6 +23,7 @@
 #include <string.h>
 #include "mozilla-prefs.h"
 #include "utils/intl.h"
+#include "kazehakase.h"
 
 
 #define DATA_KEY "KzPrefsFont::info"
@@ -50,6 +51,7 @@ static KzPrefsWinPageEntry prefs_entry =
 {
 	path:          N_("/Font"),
 	priority_hint: 0,
+	ui_level:      KZ_UI_LEVEL_ALL,
 	create:        prefs_font_create,
 	response:      prefs_font_response,
 };
@@ -543,7 +545,7 @@ prefs_font_create (void)
 	gtk_option_menu_set_menu (GTK_OPTION_MENU(option_menu), menu);
 	gtk_widget_show(menu);
 
-	g_signal_connect (option_menu, "changed",
+	g_signal_connect (G_OBJECT(option_menu), "changed",
 			  G_CALLBACK (cb_font_lang_changed), prefsui);
 
 	table = gtk_table_new(3, 2, FALSE);
@@ -822,6 +824,14 @@ prefs_font_response (GtkWidget *widget, gint response)
 		break;
 	case GTK_RESPONSE_REJECT:
 		break;
+	case KZ_RESPONSE_UI_LEVEL_BEGINNER:
+		break;
+	case KZ_RESPONSE_UI_LEVEL_MEDIUM:
+		break;
+	case KZ_RESPONSE_UI_LEVEL_EXPERT:
+		break;
+	case KZ_RESPONSE_UI_LEVEL_CUSTOM:
+		break;
 	default:
 		break;
 	}
